// The register file is a triple-port RAM
// with two read addresses and a write address.
// It is implemented with two double-port RAMs

module regfile
#(parameter DATA_WIDTH=16, parameter ADDR_WIDTH=3)
(
	input clk,
	input [(ADDR_WIDTH-1):0] ra, rb, rd,
	output [(DATA_WIDTH-1):0] qa, qb,
	input [(DATA_WIDTH-1):0] qd,
	input we
);
	double_port port1(
		.clk(clk),
		.we(we),
		.ra(ra),
		.rd(rd),
		.qa(qa),
		.qd(qd)
	);
	double_port port2(
		.clk(clk),
		.we(we),
		.ra(rb),
		.rd(rd),
		.qa(qb),
		.qd(qd)
	);
endmodule

module double_port 
#(parameter DATA_WIDTH=16, parameter ADDR_WIDTH=3)
(
	input [(DATA_WIDTH-1):0] qd,
	input [(ADDR_WIDTH-1):0] ra, rd,
	input we, clk,
	output [(DATA_WIDTH-1):0] qa
);

	// Declare the RAM variable
	reg [DATA_WIDTH-1:0] ram[0:2**ADDR_WIDTH-1];
	
	// Initialize the RAM with $readmemb.  Put the memory contents
	// in the data file.  Without this file,
	// this design will not compile.
	// See Verilog LRM 1364-2001 Section 17.2.8 for details on the
	// format of this file.

	initial
	begin
		$readmemh("registers.txt", ram);
	end

	// Variable to hold the registered read address
	reg [ADDR_WIDTH-1:0] ra_reg;
	
	always @ (posedge clk)
	begin
		// Write
		if (we)
			ram[rd] <= qd;
	end
	
	always @ (posedge clk)
		ra_reg <= ra;

	// Continuous assignment implies read returns NEW data.
	// This is the natural behavior of the TriMatrix memory
	// blocks in Single Port mode.  
	assign qa = (ra_reg? ram[ra_reg]:16'h0000);

endmodule
